<?php

namespace Drupal\Tests\rate\Functional;

use Drupal\entity_test\Entity\EntityTest;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests for the Fivestar Rate widget.
 *
 * @group rate
 */
class FivestarRateWidgetTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'entity_test',
    'rate',
  ];

  /**
   * Web users.
   *
   * @var \Drupal\user\UserInterface[]
   */
  protected $webUsers;

  /**
   * An entity to vote on.
   *
   * @var \Drupal\entity_test\Entity\EntityTest
   */
  protected $testEntity;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->testEntity = EntityTest::create(['name' => $this->randomString()]);
    $this->testEntity->save();

    // Enable voting on test entities.
    $this->config('rate.settings')
      ->set('enabled_types_bundles.entity_test', ['entity_test'])
      ->set('use_ajax', FALSE)
      ->set('widget_type', 'fivestar')
      ->save();

    $display = entity_get_display('entity_test', 'entity_test', 'default');;
    $display->setComponent('rate_vote_widget', ['weight' => 3]);
    $display->save();

    $permissions = [
      'access content',
      'cast rate vote on entity_test of entity_test',
      'view test entity',
    ];
    $this->webUsers[0] = $this->createUser($permissions);
    $this->webUsers[1] = $this->createUser($permissions);
  }

  /**
   * Tests widget caching.
   */
  public function testWidgetCaching() {
    $session = $this->assertSession();

    // Log in and vote as first user.
    $this->drupalLogin($this->webUsers[0]);
    $this->drupalGet($this->testEntity->toUrl());

    // Vote 5 stars on the item.
    $session->elementExists('css', '.rate-fivestar-btn-empty.rate-fivestar-5');
    $this->click('.rate-fivestar-btn-empty.rate-fivestar-5');
    $this->drupalGet($this->testEntity->toUrl());
    $session->elementExists('css', '.rate-fivestar-btn-filled.rate-fivestar-5');
    $session->linkExists('Undo');

    // Log in as different user, verify widget has a 5 stars rating, but should
    // still let the user vote on their own.
    $this->drupalLogin($this->webUsers[1]);
    $this->drupalGet($this->testEntity->toUrl());
    $session->elementExists('css', '.rate-fivestar-btn-filled.rate-fivestar-5');
    $session->linkNotExists('Undo');
  }

}
